#ifndef _HPG_alu_vlog_h_H_
#define _HPG_alu_vlog_h_H_

#include "systemc.h"

SC_HDL_MODULE( ALU_Vlog) {
    sc_in<bool> HCLK;
    sc_in<bool> HRESETn;
    sc_in<sc_uint<4> > HADDR;
    sc_in<sc_uint<2> > HTRANS;
    sc_in<bool> HWRITE;
    sc_in<sc_uint<3> > HSIZE;
    sc_in<sc_uint<32> > HWDATA;
    sc_in<bool> HSEL;
    sc_in<bool> HREADYin;
    sc_out<sc_uint<32> > HRDATA;
    sc_out<bool> HREADYout;
    sc_out<sc_uint<2> > HRESP;

    std::string hpg_log_lib;
    std::string hpg_module_name;
    std::string hpg_hdl_src_path;

    std::string libraryName() { return hpg_log_lib; }

    std::string moduleName() { return hpg_module_name; }

    cwr_hdlLangType hdl_language_type() { return sc_hdl_module::cwr_verilog; }

    void getVerilogSourceFiles(std::vector<std::string>& verilog_files) {
        verilog_files.push_back(hpg_hdl_src_path + std::string("../../HDL_IP/ALU/verilog/alu_top.v"));
        verilog_files.push_back(hpg_hdl_src_path + std::string("../../HDL_IP/ALU/verilog/alu.v"));
    }

    ALU_Vlog(sc_module_name name, int IDLE = 0, int BUSY = 1, int NONSEQ = 2
        , int SEQ = 3, int OK = 0, const char* hdlSrcPath="") : 
        sc_hdl_module(name), hpg_log_lib("alu_lib"), hpg_module_name("alu_top"), hpg_hdl_src_path()
        , HCLK("HCLK"), HRESETn("HRESETn"), HADDR("HADDR"), HTRANS("HTRANS"), HWRITE("HWRITE")
        , HSIZE("HSIZE"), HWDATA("HWDATA"), HSEL("HSEL"), HREADYin("HREADYin"), HRDATA("HRDATA")
        , HREADYout("HREADYout"), HRESP("HRESP") {

        if (hdlSrcPath != 0 && strlen(hdlSrcPath) != 0) {
          hpg_hdl_src_path = std::string(hdlSrcPath) + "/";
        }

        ncwr_set_hdl_param("IDLE", IDLE);
        ncwr_set_hdl_param("BUSY", BUSY);
        ncwr_set_hdl_param("NONSEQ", NONSEQ);
        ncwr_set_hdl_param("SEQ", SEQ);
        ncwr_set_hdl_param("OK", OK);

    }
};

#endif
