
//Opcode  : 0x7000_0000
//Operand1: 0x7000_0004
//Operand2: 0x7000_0008
//Result  : 0x7000_000c
`timescale 1ns/10ps

module alu_top(//input
                HCLK,
                HRESETn,
                HADDR,
				HTRANS,
				HWRITE,
				HSIZE,
				HWDATA,
				HSEL,
				HREADYin,
				//output
				HRDATA,
				HREADYout,
				HRESP
                );

input HCLK;
input HRESETn;
input HWRITE;
input HSEL;
input HREADYin;
input [1:0] HTRANS;
input [2:0] HSIZE;
input [3:0] HADDR;
input [31:0] HWDATA;

output HREADYout;
output [1:0] HRESP;
output [31:0] HRDATA;

// ===================== //
//    register domain    //
// ===================== //
/*reg HSEL_d;
reg HWRITE_d;
reg [1:0] HTRANS_d;
reg [31:0] HADDR_d;*/

reg [3:0] HADDR_d;
reg ahb_write;
reg ahb_read;

reg [1:0] Opcode;
reg [31:0] Operand1;
reg [31:0] Operand2;
//reg [31:0] Result;

// ===================== //
//      wire domain      //
// ===================== //
reg [1:0] Opcode_w;
reg [31:0] Operand1_w;
reg [31:0] Operand2_w;

wire [31:0] Result_w;

reg ahb_write_w;
reg ahb_read_w;

//HTRANS
parameter IDLE   = 2'b00;
parameter BUSY   = 2'b01;
parameter NONSEQ = 2'b10;
parameter SEQ    = 2'b11;

//HRESP
parameter OK = 2'b00;

assign HREADYout = 1'b1;
assign HRESP = OK;

alu u_alu(.Opcode(Opcode_w),
          .Operand1(Operand1_w),
	      .Operand2(Operand2_w),
		  .Result(Result_w)
		  );
		 
always @(posedge HCLK or negedge HRESETn) begin
    if(!HRESETn) begin
       ahb_write <= 0;
       ahb_read <= 0;
       HADDR_d <= 0;
    end
    else begin
       ahb_write <= ahb_write_w;
       ahb_read <= ahb_read_w;
       HADDR_d <= HADDR[3:0];
    end
end
// ahb write enable
always @(HSEL or HTRANS or HWRITE or ahb_write)
begin
     ahb_write_w = ahb_write;

     if(HSEL && (HTRANS == SEQ || HTRANS == NONSEQ) && HWRITE) begin
          ahb_write_w = 1'b1;
     end
     else begin
          ahb_write_w = 1'b0;
     end
end
// ahb read enable
always @(HSEL or HADDR[3:0] or ahb_read)
begin
     ahb_read_w = ahb_read;

     if(HSEL && HADDR[3:0] == 4'hc) begin
          ahb_read_w = 1'b1;
     end
     else begin
          ahb_read_w = 1'b0;
     end
end

// ===================== //
//       AHB write       //
// ===================== //
//assign ahb_write_w = (HSEL && (HTRANS == SEQ || HTRANS == NONSEQ) && HWRITE)?(1'b1):(1'b0);

always @(posedge HCLK or negedge HRESETn)
begin
     if(!HRESETn) begin
	      Opcode <= 0;
		  Operand1 <= 0;
		  Operand2 <= 0;
	 end
	 else begin
	      Opcode <= Opcode_w;
		  Operand1 <= Operand1_w;
		  Operand2 <= Operand2_w;
//		  Result <= Result_w;
	 end
end

always @(ahb_write or HADDR_d or HWDATA or Opcode or Operand1 or Operand2)
begin
     Opcode_w = Opcode;
	 Operand1_w = Operand1;
	 Operand2_w = Operand2;

     if(ahb_write) begin
	      case(HADDR_d)
		  4'h0: Opcode_w = HWDATA[1:0];
		  4'h4: Operand1_w = HWDATA;
		  4'h8: Operand2_w = HWDATA;
		  default: begin
		       Opcode_w = Opcode;
			   Operand1_w = Operand1;
			   Operand2_w = Operand2;
		  end
		  endcase
	 end
end

// ===================== //
//       AHB read        //
// ===================== //
//assign ahb_read = (HSEL_d && (HTRANS_d == SEQ || HTRANS_d == NONSEQ) && !HWRITE_d)?(1'b1):(1'b0);
//assign HRDATA = (ahb_read && HADDR[3:0] == 4'hc)?(Result_w):(32'hafafafaf);
//assign ahb_read_w = (HSEL && HADDR[3:0] == 4'hc)?(1'b1):(1'b0);
assign HRDATA = (ahb_read)?(Result_w):(32'hafafafaf);

endmodule
